# Copyright 2021 Marcel Konstapel
# https://www.meezenest.nl/mees
#
# This file is part of SolarGeneratorDatalogger.
# A set of programs and scripts to log all the available data from a self build solar generator.
# It runs on a Raspberry Pi Zero 1 W with Raspbian
#
# It logs data from a EPEVER Soalrge Charge Controller (via Bluetooth) and an EPEVER solar charger (via RS485-USB).
# The data is analyzed and stored in a file. The data is also available via a web browser over Wifi
# (Raspberry Pi acts as access point).
# Future plan is to implement LoRa so the data can be accessed remotely.
#
# This program reads a JSON file and returns the value of a field
#
# Usage of program:
#
#  -i, --input_file : the JSON file
#  -f, --field      : the value from this field is returned
#
# V0.1 2021-11-12
#   - First working program
#
#    SolarGeneratorDatalogger is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SolarGeneratorDatalogger is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SolarGeneratorDatalogger.  If not, see <https://www.gnu.org/licenses/>.
#
import json
import sys
import time
import getopt
import csv
import os.path

from json.decoder import JSONDecodeError
from datetime import datetime

if (len(sys.argv)<2):
    print("Usage: log_data_to_csv.py <config-file>")
else:

    # read configuration file
    try:
        with open(sys.argv[1], 'r') as myfile:
            data=myfile.read()
    except FileNotFoundError:
        print("Configuration file <"+ sys.argv[1] +"> not found.")
        sys.exit();

    # parse configuration file
    try:
        obj = json.loads(data)
        tmp = obj['liontron_bms_data_file']
        tmp2 = obj['epever_data_file']
        tmp3 = obj['csv_bms_data_file']
        tmp4 = obj['csv_solar_data_file']
    except JSONDecodeError as e:
        print("Error decoding configuration file. Not JSON format?")
        sys.exit()
    except TypeError as e:
        print("Error decoding configuration file: JSON objects not str, bytes or bytearray.")
    except KeyError:
        print("Error reading configuration file: needed fields not found.")
        sys.exit()

    # show values from config file
    print("Reading BMS data from   : " + str(obj['liontron_bms_data_file']))
    print("Reading solar data from : " + str(obj['epever_data_file']))
    print("Saving BMS csv data to  : " + str(obj['csv_bms_data_file']))
    print("Saving solar csv data to: " + str(obj['csv_solar_data_file']))
    
    # open BMS data file and extract values
    try:
        with open(obj['liontron_bms_data_file'], 'r') as myfile:
            data=myfile.read()
    except FileNotFoundError:
        print("Input file <"+ obj['liontron_bms_data_file'] +"> not found.")
        sys.exit();

    # parse data file
    try:
        bms_obj = json.loads(data)
    except JSONDecodeError as e:
        print("Error decoding BMS data file. Not JSON format?")
        sys.exit()
    except TypeError as e:
        print("Error decoding BMS data file: JSON objects not str, bytes or bytearray.")
        sys.exit()
    except KeyError:
        print("Error reading BMS data file: given field not found.")
        sys.exit()
    
    csv_data = []

    if 'system_datetime' in bms_obj:                
        csv_data.insert(0, bms_obj['system_datetime'])             
    if 'total_voltage' in bms_obj:
        csv_data.insert(1, bms_obj['total_voltage'])
    if 'current' in bms_obj:                
        csv_data.insert(2, bms_obj['current'])
    if 'remain_cap' in bms_obj:                
        csv_data.insert(3, bms_obj['remain_cap'])        
    if 'typ_cap' in bms_obj:                
        csv_data.insert(4, bms_obj['typ_cap'])        
    if 'cycles' in bms_obj:                
        csv_data.insert(5, bms_obj['cycles'])        
    if 'prod_date' in bms_obj:                
        csv_data.insert(6, bms_obj['prod_date'])        
    if 'balance_list' in bms_obj:                
        csv_data.insert(7, bms_obj['balance_list'])        
    if 'prot_status' in bms_obj:                
        csv_data.insert(8, bms_obj['prot_status'])        
    if 'softwar_version' in bms_obj:                
        csv_data.insert(9, bms_obj['softwar_version'])        
    if 'remain_cap_percent' in bms_obj:                
        csv_data.insert(10, bms_obj['remain_cap_percent'])        
    if 'fet_bits' in bms_obj:                
        csv_data.insert(11, bms_obj['fet_bits'])        
    if 'cell_count' in bms_obj:                
        csv_data.insert(12, bms_obj['cell_count'])        
    if 'ntc_count' in bms_obj:                
        csv_data.insert(13, bms_obj['ntc_count'])
    if 'temp_1' in bms_obj:                
        csv_data.insert(14, bms_obj['temp_1'])        
    if 'temp_2' in bms_obj:                
        csv_data.insert(15, bms_obj['temp_2'])        
    if 'power' in bms_obj:                
        csv_data.insert(16, bms_obj['power'])        
    if 'voltage_01' in bms_obj:                
        csv_data.insert(18, bms_obj['voltage_01'])        
    if 'voltage_02' in bms_obj:                
        csv_data.insert(19, bms_obj['voltage_02'])        
    if 'voltage_03' in bms_obj:                
        csv_data.insert(20, bms_obj['voltage_03'])        
    if 'voltage_04' in bms_obj:                
        csv_data.insert(21, bms_obj['voltage_04'])             
    
    # Now we can write the interesting values to the csv file (append)
    file_exist = os.path.exists(obj['csv_bms_data_file'])
    with open(obj['csv_bms_data_file'], 'a', encoding='UTF8') as f:

        writer = csv.writer(f)

        # Add header when file is new
        if not file_exist:
            writer.writerow(["system_datetime", "total_voltage", "current", "remain_cap", "typ_cap", "cycles", "prod_date", "balance_list", "prot_status", "softwar_version", "remain_cap_percent", "fet_bits", "cell_count", "ntc_count", "temp_1", "temp_2", "power", "voltage_01", "voltage_02", "voltage_03", "voltage_04"])

        # write the data
        writer.writerow(csv_data)
        
    # open solar data file and extract values
    try:
        with open(obj['epever_data_file'], 'r') as myfile:
            data=myfile.read()
    except FileNotFoundError:
        print("Input file <"+ obj['epever_data_file'] +"> not found.")
        sys.exit();

    # parse data file
    try:
        solar_obj = json.loads(data)
    except JSONDecodeError as e:
        print("Error decoding solar data file. Not JSON format?")
        sys.exit()
    except TypeError as e:
        print("Error decoding solar data file: JSON objects not str, bytes or bytearray.")
        sys.exit()
    except KeyError:
        print("Error reading solar data file: given field not found.")
        sys.exit()
    
    csv_data = []

    if 'system_datetime' in solar_obj:                
        csv_data.insert(0, solar_obj['system_datetime'])             
    if 'solar_voltage' in solar_obj:
        csv_data.insert(1, solar_obj['solar_voltage'])
    if 'solar_current' in solar_obj:                
        csv_data.insert(2, solar_obj['solar_current'])
    if 'solar_power' in solar_obj:                
        csv_data.insert(3, solar_obj['solar_power'])        
    if 'load_voltage' in solar_obj:                
        csv_data.insert(4, solar_obj['load_voltage'])        
    if 'load_current' in solar_obj:                
        csv_data.insert(5, solar_obj['load_current'])        
    if 'load_power' in solar_obj:                
        csv_data.insert(6, solar_obj['load_power'])        
    if 'battery_voltage' in solar_obj:                
        csv_data.insert(7, solar_obj['battery_voltage'])        
    if 'battery_current' in solar_obj:                
        csv_data.insert(8, solar_obj['battery_current'])        
    if 'battery_power' in solar_obj:                
        csv_data.insert(9, solar_obj['battery_power'])        
    if 'controller_temp' in solar_obj:                
        csv_data.insert(10, solar_obj['controller_temp'])        
    if 'ambient_temp' in solar_obj:                
        csv_data.insert(11, solar_obj['ambient_temp'])        
    if 'generated_energy_today' in solar_obj:                
        csv_data.insert(12, solar_obj['generated_energy_today'])        
    if 'controller_date_time' in solar_obj:                
        csv_data.insert(13, solar_obj['controller_date_time'])
    if 'battery_status' in solar_obj:                
        csv_data.insert(14, solar_obj['battery_status'])        
    if 'controller_status' in solar_obj:                
        csv_data.insert(15, solar_obj['controller_status'])        
    if 'load_on' in solar_obj:                
        csv_data.insert(16, solar_obj['load_on'])        
    
    # Now we can write the interesting values to the csv file (append)
    file_exist = os.path.exists(obj['csv_solar_data_file'])

    with open(obj['csv_solar_data_file'], 'a', encoding='UTF8') as f:
        writer = csv.writer(f)
        
        # Add header when file is new
        if not file_exist:
            writer.writerow(["system_datetime", "solar_voltage", "solar_current", "solar_power", "load_voltage", "load_current", "load_power", "battery_voltage", "battery_current", "battery_power", "controller_temp", "ambient_temp", "generated_energy_today", "controller_date_time", "battery_status", "controller_status", "load_on"])

        # write the data
        writer.writerow(csv_data)        
        
    print("Data from JSON files written to csv data logging files.")
